-------------------------------------------------------------------------------
--! @file       TweakeyUpdate.vhd
--! @author     Ekawat (ice) Homsirikamol
--! @brief      TweakeyUpdate for Joltik
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use work.JoltikBC_pkg.all;

entity TweakeyUpdate is
    port    (
        ii        : in  std_logic_vector(64*3           -1 downto 0);
        oo        : out std_logic_vector(64*3           -1 downto 0)        
    );
end entity TweakeyUpdate;

architecture structure of TweakeyUpdate is
    type perm_type is array(0 to 15) of integer;
    constant PERM : perm_type := (1,6,11,12,5,10,15,0,9,14,3,4,13,2,7,8);
    
    type state_type is array (0 to 2, 0 to 15) of std_logic_vector(3 downto 0);
    
    
    signal tw : state_type;
    signal hh : state_type;
    signal gg : state_type;
begin
    gWord:
    for i in 0 to 2 generate
        gLoop:
        for j in 0 to 15 generate        
            tw(i,j)       <= ii(64*(3-i)-j*4-1 downto 64*(3-i)-(j+1)*4);                    
            hh(i,PERM(j)) <= tw(i,j);
            
            gg1: if i = 0 generate
                gg(i,j)   <= hh(i,j);
            end generate;
            gg2: if i = 1 generate
                gg(i,j)   <= mul2(hh(i,j));
            end generate;
            gg4: if i = 2 generate
                gg(i,j)   <= mul4(hh(i,j));
            end generate; 
            
            oo(64*(3-i)-j*4-1 downto 64*(3-i)-(j+1)*4) <= gg(i,j);
        end generate;
    end generate;
end architecture structure;